## AVANT DE DÉBUTER
install.packages(c('gstat','lattice','rgdal','spdep','vegan','SpatialPack','ggplot2','raster','fields'))

#Note pour les utilisateurs Linux. Si l'installation de rgdal ne fonctionne pas, entrez cette commande dans le terminal:
# > sudo apt-get install libgdal-dev gdal-bin proj-bin libproj-dev

## PARTIE I - CHARGER LES DONNÉES ET LES PACKAGES NÉCESSAIRES

#Charger les packages nécessaires
library("gstat")
library("lattice")
library("rgdal")
library("spdep")
library("vegan")
library("SpatialPack")
library("ggplot2")
library("raster")
library("fields")

#Sélectionnez le fichier Arboretum_Donnees.csv sur votre ordinateur
Arbo<-read.csv(file.choose())

#Noms des variables
# Coordonnées("X","Y"), "ID", Carbone total 0-15 cm de sol("tC_015"),Carbone total 15-130 cm de sol ("tC_1530"), Carbone total couche organique ("FFtC"), Carbone sol total("tC"), Sable("Sand"), Limon("Silt"), Argile("Clay"), pH du sol ("pH"), Potassium ("K"), Phosphore("P"), Magnésium ("Mg"), Calcium ("Ca"), Minéralisation de l'azone("Min"), Surface terrière Érable rouge("RM"), Érable à sucre("SuM"), Érable argenté("SiM"), Hêtre("BE"), Chêne rouge("RO"), Bouleau blanc("WB"), Bouleau gris("GB"), Bouleau jaune("YB"), Tilleul ("BW"), Caryer cordiforme ("BH"), Caryer ovale ("SH"), Orme d'Amérique ("AE"), Frêne d'Amérique ("WA"), Frêne noir ("BA"), Ostryer de virginie ("IW"), Charme de Caroline ("MW"), Pruche ("HE") 

# Sélectionnez le fichier Arbo_mask.tif sur votre ordinateur
Arbo_mask<-readGDAL(file.choose())
xy <- Arbo[1:2]
df <- Arbo[-1:-2]
ArboSP <- SpatialPointsDataFrame(coords=xy, data=df)
proj4string(ArboSP)<-proj4string(Arbo_mask)

# Spécifie le thème de couleurs à utiliser dans SP
new_theme<-set_col_regions(terrain.colors(100))
new_theme<-set_col_regions(colorRampPalette(c("black","brown","orange","yellow","white"))(50))

# Visualiser des variables
spplot(ArboSP['Ca'])
spplot(ArboSP['tC'])
spplot(ArboSP['pH'])


##====================================================
## PARTIE 2 - CALCUL DU VARIOGRAMME ET AUTOCORRÉLATION
##====================================================

# Calcul du semi-variogramme pour le pourcentage de sable


# Nuage "variogram cloud"
Vario.SandCloud <- variogram(Sand~1,ArboSP,cloud=TRUE, cutoff=350)
plot(Vario.SandCloud)

# Moyenne par lag (variogramme 'classique')
Vario.Sand <- variogram(Sand~1,ArboSP, cutoff=350, width=50)
plot(Vario.Sand)

# Variogramme pour le carbone total
Vario.tC <- variogram(tC ~ 1, ArboSP, cutoff=350, width=50)
plot(Vario.tC)

# Choisissez le fichier GearyC.R
source(file.choose())
# Choisissez le fichier MoranI.R
source(file.choose())

# C de Geary
corgramC<-GearyC(cbind(ArboSP$X,ArboSP$Y),ArboSP$Sand,seq(0,350,50))
plot(corgramC)

# I de Moran
corgramI<-MoranI(cbind(ArboSP$X,ArboSP$Y),ArboSP$Sand,seq(0,350,50))
plot(corgramI)


##====================================================
## PARTIE 3 - SIMULATIONS NON-CONDITIONNELLES
##====================================================

# Simulations non-conditionnelles
v <- vgm(1,'Nug',0)
g.dummy <- gstat(formula = z~1, locations=ArboSP, dummy = TRUE, beta = 0,model = v, nmax = 20)
g.prdNug<-predict(g.dummy,Arbo_mask,nsim=9)
spplot(g.prdNug)

# Modèle sphérique avec une portée de 200 m
v <- vgm(1, "Sph", 200)
g.dummy <- gstat(formula = z~1, locations=ArboSP, dummy = TRUE, beta = 0,model = v, nmax = 20)
g.prdSph<-predict(g.dummy,Arbo_mask,nsim=9)
spplot(g.prdSph)

# Modèle Gaussian avec une portée de 200 m
v <- vgm(1, "Gau", 200, nug = 0.0001)
# Note, on ajoute un petit effet Nugget à la fonction Gaussienne pour des raisons mathématiques (matrice de covariance non-inversible lorsque les variances aux petites distances sont trop similaires)
g.dummy <- gstat(formula = z~1, dummy = TRUE, beta = 0,model = v, nmax = 20)
g.prdGau <- predict(g.dummy,Arbo_mask,nsim=9)
spplot(g.prdGau)

# MODÉLISATION DU VARIOGRAMME
Vario.Sand <- variogram(Sand~1, ArboSP, cutoff=350, width=50)
plot(Vario.Sand)
v <- vgm(500, "Sph", 200, nug = 250)
model.Sand = fit.variogram(Vario.Sand, model = v)
plot(Vario.Sand,model=model.Sand)

# Pour donner une idée du 'goodness of fit': 
attr(model.Sand,'SSErr') # Sommes des carrés d'écarts entre le modèle et les points. Peut servir de comparaison entre différent modèles. 

# Variogramme pour l'érable à sucre
Vario.SuM <- variogram(SuM ~ 1, ArboSP, cutoff=350, width=50)
plot(Vario.SuM)
v <- vgm(500, "Sph", 200, nug = 250)
model.SuM = fit.variogram(Vario.SuM, model = v)
plot(Vario.SuM, model=model.SuM)

# Variogramme pour le carbone total 
Vario.tC <- variogram(tC ~ 1, ArboSP, cutoff=350, width=50)
plot(Vario.tC)
v <- vgm(1, "Nug", 0)
# Effet 'pure nugget' sans composante spatiale
model.tC = fit.variogram(Vario.tC, model = v)
plot(Vario.tC, model=model.tC)

# Variogramme pour le calcium 
Vario.Ca <- variogram(Ca ~ 1, ArboSP,cutoff=350, width=50)
plot(Vario.Ca)
v <- vgm(1500000, "Sph", 200, nug = 250)
model.Ca = fit.variogram(Vario.Ca, model = v)
plot(Vario.Ca, model=model.Ca)


##====================================================
## PARTIE 4 - INTERPOLATION - KRIGEAGE - SIMULATIONS CONDITIONNELLES
##====================================================

# Inverse Distance Weighting
idwr <- idw(pH ~ 1, ArboSP, Arbo_mask,idp=2)
spplot(idwr['var1.pred'])

# Thin plate spline
tps <- Tps(xy, Arbo$pH)
ras<-raster(Arbo_mask)
spline.pH <- interpolate(ras, tps)
spline.pH <- mask(spline.pH, ras)
spplot(spline.pH)

# Simple Kriging
Vario.pH <- variogram(pH~1,ArboSP, cutoff=350, width=50)
v <- vgm(500, "Sph", 200, nug = 250)
model.pH = fit.variogram(Vario.pH, model = v)
kri <- krige(pH ~ 1, ArboSP, Arbo_mask, model = model.pH,beta=1)
spplot(kri['var1.pred'])

# Ordinary Kriging
kri <- krige(pH ~ 1, ArboSP, Arbo_mask, model = model.pH,maxdist=100)
spplot(kri['var1.pred']) # Carte prédictions
spplot(kri['var1.var']) # Carte variances

#Sauvegarder la carte krigée comme fichier Geotiff, pour utilisation dans un SIG
writeGDAL(kri['var1.pred'],'phKri.tif')

# Validation croisée en enlevant une observation à la fois
crossval.pH <- krige(pH ~ 1, ArboSP[-1,], ArboSP[1,], model = model.pH, maxdist=200)
for (i in 2:nrow(ArboSP)){
  k <- krige(pH ~ 1, ArboSP[-i,], ArboSP[i,], model = model.pH, maxdist=200)
  crossval.pH=rbind(crossval.pH,k);
}
plot(ArboSP$pH,crossval.pH$var1.pred,xlab="Valeurs mesurées", ylab="Valeurs prédites",main="pH")
cor(ArboSP$pH,crossval.pH$var1.pred) #Coefficient de corrélation entre les valeurs estimées et vraies

# SIMULATIONS CONDITIONNELLES
#Calcium
vario.Ca <- variogram(Ca ~ 1, ArboSP,cutoff=350, width=50)
plot(vario.Ca)
v <- vgm(500, "Sph", 200, nug = 250)
model.Ca = fit.variogram(vario.Ca, model = v)

Ca.sim <- krige(formula = Ca~1, ArboSP, Arbo_mask, model = model.Ca,
                nmax = 15, nsim = 9)
spplot(Ca.sim)

# pH
vario.pH <- variogram(pH ~ 1, ArboSP,cutoff=350, width=50)
plot(vario.pH)
v <- vgm(500, "Gau", 200, nug = 250)
model.pH = fit.variogram(vario.pH, model = v)

pH.sim <- krige(formula = pH~1, ArboSP, Arbo_mask, model = model.pH,
                nmax = 35, nsim = 9)
spplot(pH.sim)


##====================================================
## Partie 5 - ANALYSES BIVARIÉES
##====================================================

rm(Arbo.g)
# Note: on utilise scale ici pour standardiser les données, ce qui est pratique courante pour l'analyse multivariée. 
Arbo.g <- gstat(id="Wa", formula=scale(WA)~1, data=ArboSP, nmax = 10)
Arbo.g <- gstat(Arbo.g, "SuM", scale(SuM)~1, ArboSP, nmax = 10)
Xmodel <- vgm(0.3, "Sph", 100, nug = 0.1)
Xmodel <- vgm(0.7, "Sph", 250,add.to=Xmodel)
XVario <- variogram(Arbo.g, cutoff=300,40)
XArbo.fit = fit.lmc(XVario, Arbo.g,Xmodel,fit.lmc=TRUE)

# Variogramme croisé
plot(XVario, XArbo.fit)
xv=XVario$id=='Wa.SuM'
v1=XVario$id=='Wa'
v2=XVario$id=='SuM'

# Calcul des 'hulls' de corrélation parfaite
hulls=sqrt(XVario[v1,3]*XVario[v2,3])
hulls=cbind(hulls,-hulls)

plot(XVario[xv,2],XVario[xv,3],ylim=range(hulls))
par(new=FALSE)
lines(XVario[v1,2],hulls[,1],col='red')
lines(XVario[v1,2],hulls[,2],col='red')

# Coefficient de codispersion (valeurs entre 0 et 1 pour chaque lag)
codcoef<-XVario[xv,3]/sqrt(XVario[v1,3]*XVario[v2,3])
plot(XVario[v1,2],codcoef)

# Corrélation avec le test t modifié
#Érable à Sucre et pH
corr1<-modified.ttest(ArboSP$SuM,ArboSP$pH, coordinates(ArboSP))
corr1$ESS # Taille d'échantillon effective
print(corr1) # Notez la taille d'échantillon effective très basse, étant donné la forte structure spatiale du pH et de l'érable à sucre. 

#Érable à Sucre et Carbone total dans le sol
corr2<-modified.ttest(ArboSP$SuM,ArboSP$tC, coordinates(ArboSP))
corr2$ESS # Taille d'échantillon effective
print(corr2) # Structure spatiale moins forte pour le carbone = taille d'échantillon effective proche de N=124. 

